unit UnitBaseMDIChildForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DocIntf, Menus;

type
  TBaseMDIChildForm = class(TForm, IDoc)  //     MDI
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Sort1: TMenuItem;
    Filter1: TMenuItem;
    Command1: TMenuItem;
    Mark1: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    procedure SetCanEsc(const Value :Boolean);
    function GetCanEsc :Boolean;
    procedure SetCanClose(const Value :Boolean);
    function GetCanClose :Boolean;
  protected
    FParams :Variant;
    FCanClose: Boolean;
    FCanEsc: Boolean;
    procedure ProcessParams(const Value :Variant); virtual;
  public
    procedure SetParams(const Value :array of const);
    function ParamsIs(const Value :array of const) :Boolean;
    procedure DocInit; virtual;
    //   (  )  MDI   <Esc>
    property CanEsc :Boolean read GetCanEsc write SetCanEsc;
    //   ()  MDI 
    property CanClose :Boolean read GetCanClose write SetCanClose;
    destructor Destroy; override;
  end;
(*
      , ..
        !
*)
implementation

uses
  Misc, DB;

{$R *.dfm}

{ TBaseMDIChildForm }

destructor TBaseMDIChildForm.Destroy;
var
  I :Integer;
begin
  // ,  TDataSet      ,
  //        .
  for I := 0 to Pred(ComponentCount) do
    if Components[I] is TDataSet then TDataSet(Components[I]).Close;
  inherited;
end;

procedure TBaseMDIChildForm.DocInit;
begin
  // Document Initialization
  //   
  //   TDataSet  ..
end;

procedure TBaseMDIChildForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  //   MDI       
  with Application.MainForm do
    if MDIChildCount > 1 then MDIChildren[1].Show;
  //   ()  MDI 
  if CanClose then Action := caFree; 
end;

procedure TBaseMDIChildForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  LP :Integer;
begin
  LP := 0;
  if Shift = [] then begin
    case Key of
      //      
      //  <F3>, <F4>, <F8>
      VK_F3: LP := Ord('3');
      VK_F4: LP := Ord('4');
      VK_F8: LP := Ord('8');
      VK_ESCAPE: begin
        if CanEsc {and} {    DataSet  . .} then begin
          Key := 0;
          Close;
        end;
        Exit;
      end;
    end;
    if LP <> 0 then
      //   <Ctrl + 3>  <Ctrl + 4>  <Ctrl + 8>
      //   .  .    &3 &4 &8
      Win32Check(
        PostMessageA(Application.MainForm.Handle,
                     WM_SYSCOMMAND, SC_KEYMENU, LP));
  end;
end;

function TBaseMDIChildForm.GetCanClose: Boolean;
begin
  Result := FCanClose;
end;

function TBaseMDIChildForm.GetCanEsc: Boolean;
begin
  Result := FCanEsc;
end;

function TBaseMDIChildForm.ParamsIs(const Value: array of const): Boolean;
begin
  Result := VarEqual(FParams, VarOpenArrayToVarArray(Value));
end;

procedure TBaseMDIChildForm.ProcessParams(const Value: Variant);
begin
  // Process Params
  //   
end;

procedure TBaseMDIChildForm.SetCanClose(const Value: Boolean);
begin
  FCanClose := Value;
end;

procedure TBaseMDIChildForm.SetCanEsc(const Value: Boolean);
begin
  FCanEsc := Value;
end;

procedure TBaseMDIChildForm.SetParams(const Value: array of const);
begin
  FParams := VarOpenArrayToVarArray(Value);
end;

end.
